<?php

if(!class_exists('WC_Payment_Gateway')) return;

class WC_PK_Gateway extends WC_Payment_Gateway {

    protected $order = null;
    protected $transactionId = null;
    protected $transactionErrorMessage = null;
    protected $server = '';
    protected $secret = '';

    private $fiscal_cart = array(); //fz54 cart
    private $order_total = 0; //order total sum
    private $shipping_price = 0; //shipping price
    private $use_taxes = false;
    private $use_delivery = false;
    private $delivery_index = -1;
    private $single_item_index = -1;
    private $more_then_one_item_index = -1;
    private $order_params = NULL;
    private $discounts = array();
	
	/* инициализация плагина */
    public function __construct() {
        
		$this->id = 'paykeeper';
        $this->GATEWAYNAME = 'PayKeeper';
        $this->has_fields = false;

		load_plugin_textdomain('paykeeper', false, dirname(plugin_basename(__FILE__)) . '/languages/');
		
        $this->init_form_fields();
        $this->init_settings();

		$this->title = $this->get_option( 'title' );
		$this->icon = $this->get_option( 'icon' );
        $this->description = $this->get_option( 'description' );
        $this->server = $this->get_option( 'paykeeperserver' );
        $this->secret = $this->get_option( 'paykeepersecret' );        
        
        add_action('admin_notices', array(&$this, 'handle_admin_notice_msg'));
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
       
		add_action('woocommerce_receipt_' . $this->id, array( $this, 'receipt_page' ) );
        add_action('woocommerce_api_wc_pk_gateway', array( $this, 'paykeeper_handler' ) );
    }
	
	function handle_admin_notice_msg() {}
	
	/* Выводим настройки в административной панели*/
    public function admin_options() {
        ?>
        <h3><?php _e('PayKeeper', 'paykeeper'); ?></h3>
        <p><?php _e('Allows Payments via the PayKeeper gateway.', 'paykeeper'); ?></p>
        
        <table class="form-table">
            <?php 
            $this->generate_settings_html();
			?>
        </table>
        <?php
    }
    
	/* набор настроек */
    public function init_form_fields() {
        
		$this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'woocommerce'),
                'type' => 'checkbox',
                'label' => __('Enable PayKeeper Gateway', 'paykeeper'),
                'default' => 'yes'
            ),
            'paykeeperserver' => array(
                'title' => __('PayKeeper server URL', 'paykeeper'),
                'type' => 'text',
                'description' => __('Your PayKeeper server URL.', 'paykeeper'),
                'default' => __('', 'paykeeper')
            ),
            'paykeepersecret' => array(
                'title' => __('PayKeeper secret word', 'paykeeper'),
                'type' => 'text',
                'description' => __('Your PayKeeper secret word.', 'paykeeper'),
                'default' => __('', 'paykeeper')
            ),
			'title' => array(
                'title' => __('Title', 'woocommerce'),
                'type' => 'text',
                'description' => __('The title for this checkout option.', 'woocommerce'),
                'default' => __('PayKeeper Payment', 'paykeeper')
            ),
			'description' => array(
                'title' => __('Description', 'woocommerce'),
                'type' => 'text',
                'description' => __('The description for this checkout option.', 'woocommerce'),
                'default' => __('Payments via the PayKeeper gateway', 'paykeeper')
            ),
			'icon' => array(
                'title' => __('Icon', 'woocommerce'),
                'type' => 'text',
                'description' => __('The icon for this checkout option.', 'woocommerce'),
                'default' => __('/wp-content/plugins/woocommerce-paykeeper/visa_mastercard_wordpress.png', 'paykeeper')
            ),
			'notify_link' => array(
                'title' => __('Notify link', 'paykeeper'),
                'type' => 'text',
                'custom_attributes' => array('readonly' => 'readonly'),
                'description' => __('Copy and paste this link to your config at PayKeeper account as notify link for server-server notification.', 'paykeeper'),
                'default' => __(home_url().'/?wc-api=wc_pk_gateway', 'paykeeper')
            ),
			'success_link' => array(
                'title' => __('Success link', 'paykeeper'),
                'type' => 'text',
                'custom_attributes' => array('readonly' => 'readonly'),
                'description' => __('Copy and paste this link to your config at PayKeeper account as success payment redirect.', 'paykeeper'),
                'default' => __(home_url().'/?wc-api=wc_pk_gateway', 'paykeeper')
            ),
			'failed_link' => array(
                'title' => __('Failed link', 'paykeeper'),
                'type' => 'text',
                'custom_attributes' => array('readonly' => 'readonly'),
                'description' => __('Copy and paste this link to your config at PayKeeper account as failed payment redirect.', 'paykeeper'),
                'default' => __(home_url().'/?wc-api=wc_pk_gateway&failed', 'paykeeper')
            ),
        );
    }

	/* вывод описания платежной системы в списке доступных платежей на странице оформления заказа */
	function payment_fields(){
        if($this -> description) echo wpautop(wptexturize($this -> description));
    }
	
	/* обработка заказа, очистка корзины, переход на страницу оплаты */
    public function process_payment($order_id) {
       
		$order = new WC_Order($order_id);
		
		return array(
			'result' => 'success',
			'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, $order->get_checkout_payment_url(true)))
		);
    }
	
	/* обработка серверного-ответа на стороне сайта о подтверждении платежа */
	public function paykeeper_handler() {
		if ( ! empty( $_POST ) ) {
			
			/* у нас магазин и мы не можем обрабатывать запросы без номера заказа, хотя orderid и не обязателен */
			$order_id = $_POST['orderid'];
			if ($order_id == ""){
				echo "Error! Order id is missing";
				exit;
			}
			
			$id = $_POST['id'];
			$key = $_POST['key'];
			$email = $_POST['clientid'];
			$sum = $_POST['sum'];

			/* проверка цифровой подписи */
			if ($key != md5 ($id . $sum.$email.$order_id.$this->secret)){
				echo "Error! Hash mismatch";
				exit;
			}
			
			$order = new WC_Order($order_id);
			
			/* если заказ уже оплачен, то уведомляем, что все хорошо */
			if ( $order->has_status( 'completed' ) ) {
                $this->clearCart();
                $order->reduce_order_stock();
				echo "OK ".md5($id.$this->secret);
				exit;
			}

			/* платеж успешно совершен */
			$order->payment_complete();
			$order->add_order_note("PayKeeper payment completed");
			unset($_SESSION['order_awaiting_payment']);
			
            $this->clearCart();
            $order->reduce_order_stock();
			echo "OK ".md5($id.$this->secret);
			exit;
		}elseif(isset($_REQUEST['failed'])) self::paykeeper_failed();
		else self::paykeeper_success();
	}
	
	/* что делать при ошибке платежа */
	public function paykeeper_failed(){
		wp_redirect( get_permalink( wc_get_page_id( 'cart' ) ) );
	}
	
	/* при успешном платеже отправляем пользователя на страницу его аккаунта, последний оплаченный заказ будет самым первым в списке */
	public function paykeeper_success(){
		wp_redirect( get_permalink( wc_get_page_id( 'myaccount' ) ) );
	}
	
	/* вывод формы на странице оплаты */
 	function receipt_page($order_id){
		
		$order = new WC_Order( $order_id );

        //set order parameters
        $this->setOrderParams($order->get_total(),              //sum
                              $order->get_billing_first_name()
                              . " " .
                              $order->get_billing_last_name(),  //clientid
                              $order->get_id(),                 //orderid
                              $order->get_billing_email(),      //client_email
                              $order->get_billing_phone(),      //client_phone
                              "",                               //service_name
                              $this->server,                    //payment form url
                              $this->secret                     //secret key
        );

        //GENERATE FZ54 CART

        foreach ($order->get_items() as $order_item) {
            $taxes = array("tax" => "none", "tax_sum" => 0);
            if ((float) $order_item->get_data()["total"] <= 0)
                continue;
            $name = $order_item->get_data()["name"];
            $qty = floatval($order_item->get_data()["quantity"]);
            $sum = floatval($order_item->get_data()["total_tax"])+floatval($order_item->get_data()["total"]);
            $price = $sum/$qty;
            $tax_rate = $order_item->get_data()["total_tax"]/($order_item->get_data()["total"]/100);
            $taxes = $this->setTaxes($tax_rate);
            $this->updateFiscalCart($this->getPaymentFormType(), $name, $price, $qty, $sum, $taxes["tax"]);
        }

        //add shipping parameters to cart
        $shipping_tax_rate = 0;
        $shipping_taxes = array("tax" => "none", "tax_sum" => 0);

        $this->setShippingPrice(floatval($order->get_shipping_total())+floatval($order->get_shipping_tax()));
        $shipping_name = $order->get_shipping_method();
        //check if delivery already in cart
        if (!$this->checkDeliveryIncluded($this->getShippingPrice(), $shipping_name)
            && $this->getShippingPrice() > 0) {
            $shipping_tax_rate = floatval($order->get_shipping_tax())/(floatval($order->get_shipping_total())/100);
            $shipping_taxes = $this->setTaxes($shipping_tax_rate);
            $this->setUseDelivery(); //for precision correct check
            $this->updateFiscalCart($this->getPaymentFormType(),
                                    $shipping_name,
                                    $this->getShippingPrice(), 1, 
                                    $this->getShippingPrice(), 
                                    $shipping_taxes["tax"]);
            $this->delivery_index = count($this->getFiscalCart())-1;
        }

        //set discounts
        $this->setDiscounts((floatval($order->get_discount_total()) > 0));

        //handle possible precision problem
        $this->correctPrecision();

        $fiscal_cart_encoded = json_encode($this->getFiscalCart());

        //generate payment form
        $form = "";
        if ($this->getPaymentFormType() == "create") { //create form
            $to_hash = number_format($this->getOrderTotal(), 2, ".", "") .
                       $this->getOrderParams("clientid")     .
                       $this->getOrderParams("orderid")      .
                       $this->getOrderParams("service_name") .
                       $this->getOrderParams("client_email") .
                       $this->getOrderParams("client_phone") .
                       $this->getOrderParams("secret_key");
            $sign = hash ('sha256' , $to_hash);
            $form = '
                <h3>Сейчас Вы будете перенаправлены на страницу банка.</h3> 
                <form name="payment" id="pay_form" action="'.$this->getOrderParams("form_url").'" accept-charset="utf-8" method="post">
                <input type="hidden" name="sum" value = "'.$this->getOrderTotal().'"/>
                <input type="hidden" name="orderid" value = "'.$this->getOrderParams("orderid").'"/>
                <input type="hidden" name="clientid" value = "'.$this->getOrderParams("clientid").'"/>
                <input type="hidden" name="client_email" value = "'.$this->getOrderParams("client_email").'"/>
                <input type="hidden" name="client_phone" value = "'.$this->getOrderParams("client_phone").'"/>
                <input type="hidden" name="service_name" value = "'.$this->getOrderParams("service_name").'"/>
                <input type="hidden" name="cart" value = \''.htmlentities($fiscal_cart_encoded,ENT_QUOTES).'\' />
                <input type="hidden" name="sign" value = "'.$sign.'"/>
                <input type="submit" class="btn btn-default" value="Оплатить"/>
                </form>
                <script type="text/javascript">
                window.onload=function(){
                    setTimeout(fSubmit, 2000);
                }
                function fSubmit() {
                    document.forms["pay_form"].submit();
                }
                </script>';
        }
        else { //order form
            $payment_parameters = array(
                "clientid"=>$this->getOrderParams("clientid"), 
                "orderid"=>$this->getOrderParams('orderid'), 
                "sum"=>$this->getOrderTotal(), 
                "phone"=>$this->getOrderParams("phone"), 
                "client_email"=>$this->getOrderParams("client_email"), 
                "cart"=>$fiscal_cart_encoded);
            $query = http_build_query($payment_parameters);
            $err_num = $err_text = NULL;
            if( function_exists( "curl_init" )) { //using curl
                $CR = curl_init();
                curl_setopt($CR, CURLOPT_URL, $this->getOrderParams("form_url"));
                curl_setopt($CR, CURLOPT_POST, 1);
                curl_setopt($CR, CURLOPT_FAILONERROR, true); 
                curl_setopt($CR, CURLOPT_POSTFIELDS, $query);
                curl_setopt($CR, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($CR, CURLOPT_SSL_VERIFYPEER, 0);
                $result = curl_exec( $CR );
                $error = curl_error( $CR );
                if( !empty( $error )) {
                    $form = "<br/><span class=message>"."INTERNAL ERROR:".$error."</span>";
                    return false;
                }
                else {
                    $form = $result;
                }
                curl_close($CR);
            }
            else { //using file_get_contents
                if (!ini_get('allow_url_fopen')) {
                    $form_html = "<br/><span class=message>"."INTERNAL ERROR: Option allow_url_fopen is not set in php.ini"."</span>";
                }
                else {
                    $form = file_get_contents($server, false, $context);
                }
            }
        }
        if ($form  == "") {
            $form = '<h3>Произошла ошибка при инциализации платежа</h3><p>$err_num: '.htmlspecialchars($err_text).'</p>';
        }

		echo $form;
	}
	
    private function setOrderParams($order_total = 0, $clientid="", $orderid="", $client_email="", 
                                    $client_phone="", $service_name="", $form_url="", $secret_key="")
    {
       $this->setOrderTotal($order_total);
       $this->order_params = array(
           "sum" => $order_total,
           "clientid" => $clientid,
           "orderid" => $orderid,
           "client_email" => $client_email,
           "client_phone" => $client_phone,
           "phone" => $client_phone,
           "service_name" => $service_name,
           "form_url" => $form_url,
           "secret_key" => $secret_key,
       );
    }

    private function getOrderParams($value)
    {
        return array_key_exists($value, $this->order_params) ? $this->order_params["$value"] : False;
    }

    private function updateFiscalCart($ftype, $name="", $price=0, $quantity=0, $sum=0, $tax="none")
    {
        //update fz54 cart
        if ($ftype === "create") {
            $name = str_replace("\n ", "", $name);
            $name = str_replace("\r ", "", $name);
        }
        $this->fiscal_cart[] = array(
            "name" => $name,
            "price" => $price,
            "quantity" => $quantity,
            "sum" => $sum,
            "tax" => $tax
        );
    }

    private function getFiscalCart()
    {
        return $this->fiscal_cart;
    }

    private function setDiscounts($discount_enabled_flag)
    {
        $discount_modifier_value = 1;
        //set discounts
        if ($discount_enabled_flag) {

            if ($this->getFiscalCartSum(false) > 0)
                $discount_modifier_value = ($this->getOrderTotal() - $this->getShippingPrice())/$this->getFiscalCartSum(false);

            if ($discount_modifier_value < 1) {
                for ($pos=0; $pos<count($this->getFiscalCart()); $pos++) {//iterate fiscal cart without shipping
                    if ($pos != $this->delivery_index) {
                        $this->fiscal_cart[$pos]["sum"] *= $discount_modifier_value;
                        $this->fiscal_cart[$pos]["price"] = $this->fiscal_cart[$pos]["sum"]/$this->fiscal_cart[$pos]["quantity"];
                    }
                }
            }
        }
    }

    private function correctPrecision()
    {
        //handle possible precision problem
        $fiscal_cart_sum = $this->getFiscalCartSum(true);
        $total_sum = $this->getOrderTotal();
        $diff_sum = $total_sum - $fiscal_cart_sum;
        //debug_info
        //echo "\ntotal: $total_sum - cart: $fiscal_cart_sum - diff: $diff_sum";
        if ($this->getUseDelivery()) { //delivery is used
            $this->correctPriceOfCartItem($diff_sum, count($this->fiscal_cart)-1);
        }
        else {
            if ($this->single_item_index >= 0) { //we got single cart element
                $this->correctPriceOfCartItem($diff_sum, $this->single_item_index);
            }
            else if ($this->more_then_one_item_index >= 0) { //we got cart element with more then one quantity
                $this->splitCartItem($this->more_then_one_item_index);
                //add diff_sum to the last element (just separated) of fiscal cart
                $this->correctPriceOfCartItem($diff_sum, count($this->fiscal_cart)-1);
            }
            else { //we only got cart elements with less than one quantity
                $modify_value = ($diff_sum > 0) ? $total_sum/$fiscal_cart_sum : $fiscal_cart_sum/$total_sum;
                for ($pos=0; $pos<count($this->getFiscalCart()); $pos++) {
                    $this->fiscal_cart[$pos]["sum"] *= $modify_value;
                    $this->fiscal_cart[$pos]["price"] = $this->fiscal_cart[$pos]["sum"]/$this->fiscal_cart[$pos]["quantity"];
                }
            }
        }
    }

    private function setOrderTotal($value)
    {
        $this->order_total = $value;
    }

    private function getOrderTotal()
    {
        return $this->order_total;
    }

    private function setShippingPrice($value)
    {
        $this->shipping_price = $value;
    }

    private function getShippingPrice()
    {
        return $this->shipping_price;
    }

    private function getPaymentFormType()
    {
        if (strpos($this->order_params["form_url"], "/order/inline") == True)
            return "order";
        else
            return "create";
    }

    private function setUseTaxes()
    {
        $this->use_taxes = True;
    }

    private function getUseTaxes()
    {
        return $this->use_taxes;
    }

    private function setUseDelivery()
    {
        $this->use_delivery = True;
    }

    private function getUseDelivery()
    {
        return $this->use_delivery;
    }

    private function setTaxes($tax_rate, $zero_value_as_none = true)
    {
        $taxes = array("tax" => "none", "tax_sum" => 0);
        switch(number_format($tax_rate, 0, ".", "")) {
            case 0:
                if (!$zero_value_as_none) {
                    $taxes["tax"] = "vat0";
                }
                break;
            case 10:
                $taxes["tax"] = "vat10";
                break;
            case 18:
                $taxes["tax"] = "vat18";
                break;
            case 20:
                $taxes["tax"] = "vat20";
                break;
        }
        return $taxes;
    }

    private function checkDeliveryIncluded($delivery_price, $delivery_name) {
        $index = 0;
        foreach ($this->getFiscalCart() as $item) {
            if ($item["name"] == $delivery_name
                && $item["price"] == $delivery_price
                && $item["quantity"] == 1) {
                $this->delivery_index = $index;
                return true;
            }
            $index++;
        }
        return false;
    }

    private function getFiscalCartSum($delivery_included) {
        $fiscal_cart_sum = 0;
        $index = 0;
        foreach ($this->getFiscalCart() as $item) {
            if (!$delivery_included && $index == $this->delivery_index)
                continue;
            $fiscal_cart_sum += $item["sum"];
            $index++;
        }
        return $fiscal_cart_sum;
    }

    private function showDebugInfo($obj_to_debug)
    {
        echo "<pre>";
        var_dump($obj_to_debug);
        echo "</pre>";
    }

    private function clearCart()
    {
		//clear cart
        global $woocommerce;
        $woocommerce->cart->empty_cart();
    }

    private function correctPriceOfCartItem($corr_price_to_add, $item_position)
    {
        $item_sum = 0;
        $this->fiscal_cart[$item_position]["price"] += $corr_price_to_add;
        $item_sum = $this->fiscal_cart[$item_position]["price"]*$this->fiscal_cart[$item_position]["quantity"];
        $this->fiscal_cart[$item_position]["sum"] = $item_sum;
    }

    private function splitCartItem($cart_item_position)
    {
        $item_sum = 0;
        $item_price = 0;
        $item_quantity = 0;
        $item_price = $this->fiscal_cart[$cart_item_position]["price"];
        $item_quantity = $this->fiscal_cart[$cart_item_position]["quantity"]-1;
        $this->fiscal_cart[$cart_item_position]["quantity"] = $item_quantity; //decreese quantity by one
        $this->fiscal_cart[$cart_item_position]["sum"] = $item_price*$item_quantity; //new sum
        //add one cart item to the end of cart
        $this->updateFiscalCart(
            $this->fiscal_cart[$cart_item_position]["name"],
            $item_price, 1, $item_price,
            $this->fiscal_cart[$cart_item_position]["tax"]);
    }
}
